@@ -13,6 +13,7 @@ import requests
 import rasterio
 from datetime import datetime
 import math
+import tifffile
 from shapely.geometry import Polygon
 from rasterio.features import rasterize
 from geopy.distance import geodesic
@@ -25,7 +26,7 @@ from requests_toolbelt import MultipartEncoder
 import warnings
 import threading
 
-__version__ = '1.2.7'
+__version__ = '1.2.14'
 url = 'https://api.ellipsis-drive.com/v1'
 
 s = requests.Session()
@@ -43,14 +44,14 @@ def logIn(username, password):
         token = 'Bearer ' + token
         return(token)
 
-def metadata(projectId, token = None):
+def metadata(projectId, includeDeleted=False, token = None):
     mapId = projectId
     if token == None:
         r = s.post(url + '/metadata',
                          json = {"mapId":  mapId})
     else:
         r = s.post(url + '/metadata', headers = {"Authorization":token},
-                         json = {"mapId":  mapId})
+                         json = {"mapId":  mapId, 'includeDeleted':includeDeleted})
     if int(str(r).split('[')[1].split(']')[0]) != 200:
         raise ValueError(r.text)
     
@@ -69,7 +70,7 @@ def getShapes(name= None, fuzzyMatch = False, favorite = None, access = ['subscr
     if str(type(favorite)) != str(type(None)):
         body['favorite'] = name
     
-    if str(type(userId)) == str(type(None)):
+    if str(type(userId)) != str(type(None)):
         body['userId'] = userId
     if str(type(hashtag)) != str(type(None)):
         body['hashtag'] = hashtag
@@ -366,6 +367,9 @@ def geometryEdit(shapeId, layerId, geometryIds, features, token, zoomlevels = No
     if 'attribution' in features.columns:
         del features['attribution']
 
+    if str(type(zoomlevels)) != str(type(None)):
+        zoomlevels = [int(z) for z in zoomlevels]
+
 
     mapId = shapeId
     if not str(type(features)) ==  "<class 'geopandas.geodataframe.GeoDataFrame'>":
@@ -382,13 +386,13 @@ def geometryEdit(shapeId, layerId, geometryIds, features, token, zoomlevels = No
         
         features_sub =features_sub.to_json(na='drop')
         features_sub = json.loads(features_sub)
-        
+
         if str(type(zoomlevels)) != str(type(None)):
-            zoomlevels = list(zoomlevels)
             changes = [{'geometryId':x[0] , 'newProperties':x[1]['properties'], 'newGeometry':x[1]['geometry'], 'newZoomlevels':zoomlevels} for x in zip(geometryIds_sub, features_sub['features'])]
         else:
             changes = [{'geometryId':x[0] , 'newProperties':x[1]['properties'], 'newGeometry':x[1]['geometry']} for x in zip(geometryIds_sub, features_sub['features'])]
             
+            
         body = {"mapId":  mapId, "layerId":layerId, 'changes':changes}
         retried = 0
         while retried <= 20:
@@ -409,19 +413,6 @@ def geometryEdit(shapeId, layerId, geometryIds, features, token, zoomlevels = No
         loadingBar(i*1000 + len(indices_sub),features.shape[0])
 
 
-def geometryMove(shapeId, geometryIds, oldLayerId, newLayerId, token):
-    mapId = shapeId
-    geometryIds = list(geometryIds)
-    chunks_ids = chunks(geometryIds)
-    N=0
-    for chunk_ids in chunks_ids:
-        r = s.post(url + '/geometry/move', headers = {"Authorization":token},
-                         json = {"mapId":  mapId, 'geometryIds':chunk_ids, 'newLayerId':newLayerId, "oldLayerId": oldLayerId})
-        if int(str(r).split('[')[1].split(']')[0]) != 200:
-            raise ValueError(r.text)
-        loadingBar(N*3000 + len(chunk_ids), len(geometryIds))
-        N=N+1
-
 def geometryChangelog(shapeId, layerId, limit = 100, userId = None, pageStart = None, actions = ['add', 'delete', 'recover', 'move'], token = None):
     changes = []
     keepGoing = True
@@ -463,6 +454,9 @@ def geometryAdd(shapeId, layerId, features, token, zoomlevels=None):
         raise ValueError('Please provide CRS for the geopandas dataframe or translate to WGS84 coordinates')
     else:
         features = features.to_crs({'init': 'epsg:4326'})
+
+    if str(type(None)) != str(type(zoomlevels)):
+        zoomlevels = [int(z) for z in zoomlevels]
     
     firstTime = metadata(projectId = mapId, token = token)['geometryLayers']
     firstTime = [x for x in firstTime if x['id'] == layerId]
@@ -643,12 +637,13 @@ def messageImage(shapeId, layerId, messageId, token = None):
 
 
 
-def rasterAggregated(mapId, timestamp, geometry, approximate = True, token = None):
+def rasterAggregated(mapId, timestamps, geometry, approximate = True, token = None):
     sh = gpd.GeoDataFrame({'geometry':[geometry]})
     geometry =sh.to_json(na='drop')
     geometry = json.loads(geometry)
     geometry = geometry['features'][0]['geometry']
-    body = {'mapId':mapId, 'timestamp':timestamp, 'geometry':geometry, 'approximate':approximate}
+    timestamps = list(timestamps)
+    body = {'mapId':mapId, 'timestamps':timestamps, 'geometry':geometry, 'approximate':approximate}
     if token ==None:
         r = s.post(url +  '/raster/aggregated',
                      json = body )        
@@ -662,7 +657,7 @@ def rasterAggregated(mapId, timestamp, geometry, approximate = True, token = Non
 
 def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom = None, num_bands = None, bounds=None, downsample = False, width = None, height = None, threads = 1, token = None):
     dtype = 'float32'
-
+    
     if str(type(bounds)) != str(type(None)):
       xMin,yMin,xMax,yMax = bounds.bounds  
     elif str(type(xMin)) == "<class 'NoneType'>" or str(type(xMax)) == "<class 'NoneType'>" or str(type(yMin)) == "<class 'NoneType'>" or str(type(yMax)) == "<class 'NoneType'>" :
@@ -689,7 +684,7 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
 
         bbox = {'xMin':xMin, 'xMax':xMax, 'yMin':yMin , 'yMax':yMax}
         body = {'mapId':mapId, 'timestamp' : timestamp, 'mapId':mapId, 'bounds':bbox, 'width':width, 'height':height}
-
+        print(body)
         if str(type(token)) == str(type(None)):
             r = s.post(url + '/raster/raw',
                          json = body )        
@@ -701,10 +696,8 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
         if int(str(r).split('[')[1].split(']')[0]) != 200:
                 raise ValueError(r.text)
         else:
-                 with MemoryFile(r.content) as memfile:
-                     with memfile.open() as dataset:
-                         r_total = dataset.read()
-        r_total = np.transpose(r_total, (1,2,0) )
+                r_total = tifffile.imread(BytesIO(r.content))
+                r_total = np.transpose(r_total, [2,0,1])
 
     else:
             if str(type(num_bands)) == str(type(None)):
@@ -724,17 +717,17 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
             max_x_osm_precise =   (xMax +180 ) * 2**zoom / 360
             max_y_osm_precise = 2**zoom / (2* math.pi) * ( math.pi - math.log( math.tan(math.pi / 4 + yMin/360 * math.pi  ) ) ) 
             min_y_osm_precise = 2**zoom / (2* math.pi) * ( math.pi - math.log( math.tan(math.pi / 4 + yMax/360 * math.pi  ) ) )
-        
+                        
             min_x_osm =  math.floor(min_x_osm_precise )
             max_x_osm =  math.floor( max_x_osm_precise)
             max_y_osm = math.floor( max_y_osm_precise)
             min_y_osm = math.floor( min_y_osm_precise)
-        
+            
             x_tiles = np.arange(min_x_osm, max_x_osm+1)
             y_tiles = np.arange(min_y_osm, max_y_osm +1)
-
+            
             r_total = np.zeros((256*(max_y_osm - min_y_osm + 1) ,256*(max_x_osm - min_x_osm + 1),num_bands), dtype = dtype)
-
+            
             tiles = []            
             for tileY in y_tiles:
                 for tileX in x_tiles:
@@ -747,9 +740,9 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
                         tileY = tile[1]
                         x_index = tileX - min_x_osm
                         y_index = tileY - min_y_osm
-    
-                        r = s.get(url + '/tileService/' + mapId + '/' + str(timestamp) + '/data/' + str(zoom) + '/' + str(tileX) + '/' + str(tileY) + token_inurl ,
-                                     timeout = 10 )
+                        
+                        url_req = url + '/tileService/' + mapId + '/' + str(timestamp) + '/data/' + str(zoom) + '/' + str(tileX) + '/' + str(tileY) + token_inurl
+                        r = s.get(url_req , timeout = 10 )
     
     
                         if int(str(r).split('[')[1].split(']')[0]) == 403:
@@ -757,14 +750,9 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
                         if int(str(r).split('[')[1].split(']')[0]) != 200:
                                 r = np.zeros((num_bands,256,256))
                         else:
-                                 with MemoryFile(r.content) as memfile:
-                                     with memfile.open() as dataset:
-                                         r = dataset.read()
-                        r = np.transpose(r, (1,2,0) )
-                        
-                        bands = list(np.arange(num_bands-1))
-                        bands.append(r.shape[2]-1)
-                        r_total[y_index*256:(y_index+1)*256,x_index*256:(x_index+1)*256, : ] = r[:,:, bands]
+                            r = tifffile.imread(BytesIO(r.content))
+                        r = np.transpose(r, [1,2,0])
+                        r_total[y_index*256:(y_index+1)*256,x_index*256:(x_index+1)*256, : ] = r
                         loadingBar(N, len(tiles))
                         N = N + 1
                     
@@ -780,9 +768,9 @@ def rasterRaw(mapId, timestamp, xMin= None,xMax= None,yMin=None, yMax=None, zoom
                 pr.join()                           
                 
             min_x_index = int(round((min_x_osm_precise - min_x_osm)*256))
-            max_x_index = int(round((max_x_osm_precise- min_x_osm)*256))
+            max_x_index = max(int(round((max_x_osm_precise- min_x_osm)*256)), min_x_index + 1 )
             min_y_index = int(round((min_y_osm_precise - min_y_osm)*256))
-            max_y_index = int(round((max_y_osm_precise- min_y_osm)*256))
+            max_y_index = max(int(round((max_y_osm_precise- min_y_osm)*256)), min_y_index + 1)
             
             r_total = r_total[min_y_index:max_y_index,min_x_index:max_x_index,:]
  
@@ -1147,10 +1135,10 @@ def activateTimestamp(mapId, timestampId, active, token):
         raise ValueError(r.text)
 
 
-def removeTimestamp(mapId, timestampId, token):
+def removeTimestamp(mapId, timestampNumber, token, revert = False, hard = False):
 
-    r = s.post(url + '/settings/projects/reschedule', headers = {"Authorization":token},
-                 json = {"mapId":  mapId, "toRemove":[timestampId]})
+    r = s.post(url + '/settings/projects/deleteTimestamp', headers = {"Authorization":token},
+                 json = {"mapId":  mapId, "timestamp":timestampNumber, 'revert':revert, 'hard':hard})
 
     if int(str(r).split('[')[1].split(']')[0]) != 200:
         raise ValueError(r.text)
@@ -1252,18 +1240,43 @@ def deleteProperty(shapeId, layerId, propertyName, token, revert = False):
 
 
 
-def ShapeLayerIndex(shapeId, layerId, token, idProperty=None, valueProperty=None, dateProperty=None ):
+def ShapeLayerIndex(shapeId, layerId, token, filterProperties = [], idProperty = None):
+        
+    if type(filterProperties) != type([]):
+        raise ValueError('filterProperties must be a list with property names')
+
+    toAdd = [ {'property': p, 'id':False } for p in filterProperties]
+
+    if str(type(idProperty)) != str(type(None)):
+        if str(type(idProperty)) != str(type('hi')):
+            raise ValueError('idProperty must be of type string')
+        toAdd = toAdd + [{'property': idProperty, 'id':True }]
 
 
+    
+    layer = [layer for layer in metadata(shapeId, True, token)['geometryLayers'] if layer['id'] == layerId]
+    if len(layer) == 0:
+        raise ValueError('layerId not found in this shape')
+        
+    layer = layer[0]
+    properties = layer['properties']    
+    currentFilterProperties = [p for p in properties if 'filtering' in p.keys() ]
+
+    names = [n['property'] for n in toAdd]
+    toRemove = list( set(currentFilterProperties).difference(set(names)) )    
+    
     body = {"mapId":shapeId,"layerId":layerId}
     
-    if str(type(None)) != str(type(idProperty)):
-        body['idProperty'] = idProperty
-    if str(type(None)) != str(type(valueProperty)):
-        body['valueProperty'] = valueProperty
-    if str(type(None)) != str(type(dateProperty)):
-        body['dateProperty'] = dateProperty
+    
+    if len(toAdd) >0:
+        body['propertiesToAdd'] = toAdd
+        
+    if len(toRemove) > 0:
+        body['propertiesToRemove'] = toRemove
 
+    if len(toRemove) == 0 and len(toAdd) ==0:
+        raise ValueError('The current properties are already the current filter properties')
+    
     r = s.post(url + '/settings/geometryLayers/reIndex', headers = {"Authorization":token},
                      json = body)
     
